function [mujedna,munula,lambda,Sigma,logver,pociter,tolerkonec,normal]=MNDest(X,tolerance,rovnat,poclambda)
  % funkce pro vypocet maximalne verohodnych odhadu v modelu
  % smesi normalnich rozdeleni (MND) pomoci EM algotitmu
  
  % nutno zadat pouze pocatecni lambda
  % pocatecni odhady se potom pocitaji automaticky jako
  % pocmujedna a pocmunula prumer prvni, resp. druhe pulky dat;
  % pocsigma bezne s pouzitim pocmujedna a pocmunula
  % X...matice dat, kde znaky u jednotlivych objektu jsou v radcich
  
  % X je tedy typu n x p
  % tolerance...kdy skoncit
  % rovnat...podle ktereho sloupce matice X se maji data setridit
  
  % logver...konecna hodnota logaritmicke verohodnosti
  % pociter...pocet iteraci
  % normal...1-pokud pociter<maximalni pocet iteraci (maxiter)
  %          0-pokud pociter=maximalni pocet iteraci (maxiter)
  %           potom se konci bez ohledu na zvolenou toleranci
  
  maxiter=500;  % maximalni pocet iteraci
  n=size(X,1);  % pocet radku X = pocet objektu
  p=size(X,2);  % pocet sloupcu X = pocet vysvetlujicich velicin
  
  if rovnat>0
     pomocnamatice=-X;
     pomocnamatice=sortrows(pomocnamatice,rovnat);
     X=-pomocnamatice;  % setrideni dat podle zadaneho sloupce
                        % srovna se to od nejvetsich hodnot po nejmensi   
                        % predpoklada se, ze velkym hodnotam odpovida skupina s Y=1      
  end;   
  
  % vypocty pocatecnich odhadu
  njedna=round(n*poclambda);
  nnula=n-njedna;  % rozdeleni dat na dve skoropoloviny
  Xjedna=X(1:njedna,:);
  Xnula=X(njedna+1:n,:); % matice rozdelenych dat
  poclambda=poclambda % aby se to vypsalo
  pocmujedna=(1/njedna)*(sum(Xjedna))'
  pocmunula=(1/nnula)*(sum(Xnula))'
  soucetjedna=zeros(p);
  for i=1:njedna
     soucetjedna=soucetjedna+(((Xjedna(i,:))'-pocmujedna)*((Xjedna(i,:))'-pocmujedna)');
  end;   
  soucetnula=zeros(p);
  for i=1:nnula
     soucetnula=soucetnula+(((Xnula(i,:))'-pocmunula)*((Xnula(i,:))'-pocmunula)');
  end;   
  pocsigma=(1/n)*(soucetjedna+soucetnula)

  % nulta iterace
  mujedna = pocmujedna;    % typu p x 1
  munula = pocmunula;      % typu p x 1
  lambda = poclambda;      % cislo  
  Sigma = pocsigma;        % typu p x p 
  
  pociter = 1;
  rozdil=100000;  % aby to poprve bylo vetsi nez tolerance
  
  % vypocet pocatecni log verohodnosti   
  logverstara=0;
  for i=1:n
       logverstara=logverstara+log(lambda*norhust(mujedna,Sigma,(X(i,:))')+(1-lambda)*norhust(munula,Sigma,(X(i,:))'));
  end;   
  
  fprintf(1,'iterace    logverohodnost     rozdil\n')
  
  logverohodnost=logverstara;  % aby se vypsala pocatecni log verohodnost
  iterace=0;
  rozdilll=0;
  uuu=[iterace;logverohodnost;rozdilll];
  fprintf(1,'%3.0f        %12.5f      %12.7f\n',uuu)

  
  % iteracni proces
  while ((pociter<(maxiter+1))&(abs(rozdil)>tolerance))  % iteruje se do te doby, dokud 
           % se neprekroci maximalni pocet iteraci nebo narust logaritmicke verohodnosti 
           % neni mensi nez tolerance
           
     % vypocet vah      
     for i=1:n
        wjed(i,1)=(lambda*norhust(mujedna,Sigma,(X(i,:))'))/(lambda*norhust(mujedna,Sigma,(X(i,:))')+(1-lambda)*norhust(munula,Sigma,(X(i,:))'));    
        wnula(i,1)=1-wjed(i,1);
     end;  
     
     % novy odhad lambda
     lambda=(1/n)*sum(wjed);  
     
     % vypocet noveho mujedna
     pomoc=zeros(p,1);
     for i=1:n
        pomoc=pomoc+(wjed(i,1)*(X(i,:))');
     end;  % pro vypocet noveho mujedna
     mujedna=(1/sum(wjed))*pomoc; % novy odhad mujedna
     
     % vypocet noveho munula
     pomoc=zeros(p,1);
     for i=1:n
        pomoc=pomoc+(wnula(i,1)*(X(i,:))');
     end;  % pro vypocet noveho mujedna
     munula=(1/sum(wnula))*pomoc; % novy odhad munula
     
     % vypocet nove matice Sigma
     pomocjedna=zeros(p);   % nulova matice typu p x p
     pomocnula=zeros(p);    % nulova matice typu p x p
     for i=1:n
        pomocjedna=pomocjedna+(wjed(i,1)*(X(i,:)'-mujedna)*(X(i,:)'-mujedna)');
     end;  % pro vypocet nove Sigma (prvni pulka scitance)
     for i=1:n
        pomocnula=pomocnula+(wnula(i,1)*(X(i,:)'-munula)*(X(i,:)'-munula)');
     end;  % pro vypocet nove Sigma (druha pulka scitance)
     Sigma=(1/n)*(pomocjedna+pomocnula);   % novy odhad Sigma    
     
     % vypocet nove logaritmicke verohodnosti
     logvernova=0;
     for i=1:n
        logvernova=logvernova+log(lambda*norhust(mujedna,Sigma,(X(i,:))')+(1-lambda)*norhust(munula,Sigma,(X(i,:))'));
     end;  
     
     rozdil=logvernova-logverstara;
     pociter=pociter+1;
     logverstara=logvernova;
     
     % vypis mezivysledku     
     logverohodnost=logverstara; 
     iterace=pociter-1;
     rozdil=rozdil;
     uuu=[iterace;logverohodnost;rozdil];
     fprintf(1,'%3.0f        %12.5f      %12.7f\n',uuu)

  end;   % konec whilu
  
  % vysledky
  logver=logvernova;
  normal=(pociter<(maxiter+1));
  pociter=pociter-1;
  tolerkonec=rozdil;
     
        
  

